
# this script was generated by openmm-builder. to customize it further,
# you can save the file to disk and edit it with your favorite editor.
##########################################################################

from __future__ import print_function
from simtk.openmm import app
import simtk.openmm as mm
from simtk import unit
from sys import stdout, exit
#import parmed as pmd

temperature = 184*unit.kelvin
pressure = 1.0*unit.atmospheres

app.topology.Topology.loadBondDefinitions('residues.xml')
pdb = app.PDBFile('benzene.pdb')
forcefield = app.ForceField('sapt_benzene.xml')
#parm = pmd.load_file('water.pdb','sapt_water.xml')

# Add drude particle positions to the pdb file
model = app.modeller.Modeller(pdb.topology, pdb.positions)
model.addExtraParticles(forcefield)
model_topology = model.getTopology()
model_positions = model.getPositions()

# Create system from force field
system = forcefield.createSystem(model_topology, 
                #nonbondedMethod=app.PME, 
                #nonbondedMethod=app.CutoffPeriodic,
                nonbondedMethod=app.NoCutoff,
                nonbondedCutoff=1.2*unit.nanometers, constraints=None,
                #rigidWater=True, 
                ewaldErrorTolerance=0.0005)



def forcegroupify(system):
    forcegroups = {}
    for j in range(system.getNumForces()):
        force = system.getForce(j)
        force.setForceGroup(j)
        forcegroups[force] = j
    return forcegroups

def getEnergyDecomposition(context, forcegroups):
    energies = {}
    for f, j in forcegroups.items():
        energies[f] = context.getState(getEnergy=True,
groups=2**j).getPotentialEnergy()
    return energies

def getForceDecomposition(context, forcegroups):
    forces = {}
    for f, j in forcegroups.items():
        forces[f] = context.getState(getForces=True,
groups=2**j).getForces()
    return forces


integrator = mm.LangevinIntegrator(
                # system temperature:
                temperature, 2.0/unit.picoseconds, 
                # drude temperature:
                #1*unit.kelvin, 200.0/unit.picoseconds, 
                # timestep:
                0.1*unit.femtoseconds)

platform = mm.Platform.getPlatformByName('CPU')
#
simulation = app.Simulation(model_topology, system, integrator, platform)
simulation.context.setPositions(model_positions)

customHbondForce = [f for f in system.getForces() if type(f) == mm.CustomHbondForce ][0]
print('num acceptors:',customHbondForce.getNumAcceptors())
print('num donors:',customHbondForce.getNumDonors())
print('num exclusions:',customHbondForce.getNumExclusions())
print('energy expression:',customHbondForce.getEnergyFunction())
print()

customNonbondForce = [f for f in system.getForces() if type(f) == mm.CustomNonbondedForce ][0]
print('num particles:',customNonbondForce.getNumParticles())
print('num exclusions:',customNonbondForce.getNumExclusions())
print('energy expression:',customNonbondForce.getEnergyFunction())
print()

forcegroups = forcegroupify(system)
energy = getEnergyDecomposition(simulation.context,forcegroups)
for k,v in energy.items():
    print(k,v)
print('----')
## forces = getForceDecomposition(simulation.context,forcegroups)
## for k,v in forces.items():
##     print(k)
##     for line in v:
##         print(line)
## print('----')
position = simulation.context.getState(getPositions=True).getPositions()
app.PDBFile.writeFile(simulation.topology, position, open('beforemin.pdb', 'w'))

exit()

print('Minimizing...')
position = simulation.context.getState(getPositions=True).getPositions()
energy = simulation.context.getState(getEnergy=True).getPotentialEnergy()
app.PDBFile.writeFile(simulation.topology, position, open('beforemin.pdb', 'w'))
for line in position:
    print(line)
print(energy)
forcegroups = forcegroupify(system)
energy = getEnergyDecomposition(simulation.context,forcegroups)
for k,v in energy.items():
    print(k,v)
print('----')

mm.LocalEnergyMinimizer.minimize(simulation.context) 
position = simulation.context.getState(getPositions=True).getPositions()
energy = simulation.context.getState(getEnergy=True).getPotentialEnergy()
for line in position:
    print(line)
print(energy)
forcegroups = forcegroupify(system)
energy = getEnergyDecomposition(simulation.context,forcegroups)
for k,v in energy.items():
    print(k,v)
print('----')
app.PDBFile.writeFile(simulation.topology, position, open('min.pdb', 'w'))

simulation.minimizeEnergy(0.01*unit.kilojoule/unit.mole)
position = simulation.context.getState(getPositions=True).getPositions()
energy = simulation.context.getState(getEnergy=True).getPotentialEnergy()
for line in position:
    print(line)
print(energy)
app.PDBFile.writeFile(simulation.topology, position, open('min2.pdb', 'w'))

## forcegroups = forcegroupify(system)
## energy = getEnergyDecomposition(simulation.context,forcegroups)
## for k,v in energy.items():
##     print(k,v)
## 
## exit()

## 
## simulation.context.setVelocitiesToTemperature(temperature)
## print('Equilibrating...')
## ## simulation.reporters.append(app.StateDataReporter(stdout, 1, step=True, 
## ##         potentialEnergy=True, kineticEnergy=True, temperature=True,
## ##         density=True, speed=True, separator='\t'))
## simulation.step(1000)
print('taking step')

simulation.step(1)
position = simulation.context.getState(getPositions=True).getPositions()
energy = simulation.context.getState(getEnergy=True).getPotentialEnergy()
app.PDBFile.writeFile(simulation.topology, position, open('min_step.pdb', 'w'))
exit()

simulation.reporters.append(app.PDBReporter('output.pdb', 1000))
simulation.reporters.append(app.StateDataReporter('output.out', 100, step=True, 
#simulation.reporters.append(app.StateDataReporter(stdout, 1, step=True, 
        potentialEnergy=True, kineticEnergy=True, temperature=True,
        density=True, speed=True, separator='\t'))

print('Running Production...')
for i in range(100):
    simulation.step(10000)
print('Done!')


